import base64

import requests

from aria2jrpc.models import (
    Aria2CommonOptions,
    Aria2GlobalOptions,
    Aria2InputOptions,
    HowPosition,
)


class Aria2JRPC:
    def __init__(
        self,
        url: str,
        secret: str = "",
        global_options: Aria2GlobalOptions = Aria2GlobalOptions(),
    ) -> None:
        self._base_url = url + "/jsonrpc"
        self._secret = secret
        self._id = 1
        self._global_option = global_options
        self.change_global_option(global_options)

    def _get(
        self,
        method: str,
        params: list | None = [],
        id: str | None = None,
    ) -> dict:
        if not id:
            id = self._id
            self._id += 1
        params = [f"token:{self._secret}"] + params
        json_data = {
            "jsonrpc": "2.0",
            "method": method,
            "id": id,
            "params": params,
        }
        res = requests.post(
            self._base_url,
            json=json_data,
        )
        json_res = res.json()
        if error := json_res.get("error"):
            raise Exception(error["message"])
        return json_res

    def _download(
        self,
        method: str,
        options: Aria2InputOptions,
        position: int | None,
        params: list = [],
        id: str | None = None,
    ) -> dict:
        params.append(options.to_dict())
        if position is not None:
            params.append(position)
        return self._get(method, params, id)

    def add_uri(
        self,
        *uris: str,
        id: int | None = None,
        options: Aria2InputOptions | None = Aria2InputOptions(),
        position: int | None = None,
    ) -> str:
        return self._download(
            "aria2.addUri",
            id=id,
            params=[uris],
            options=options,
            position=position,
        )["result"]

    def add_torrent(
        self,
        *torrent_files_pathes: str,
        id: int | None = None,
        options: Aria2InputOptions | None = Aria2InputOptions(),
        position: int | None = None,
    ) -> str:
        torrents = []
        for torrent_file_path in torrent_files_pathes:
            with open(torrent_file_path, "r") as tf:
                base64.b64encode(tf.read())
        return self._download(
            "aria2.addTorrent",
            id=id,
            params=torrents,
            options=options,
            position=position,
        )["result"]

    def add_meta_link(
        self,
        *meta_file_pathes: str,
        id: int | None = None,
        options: Aria2InputOptions | None = Aria2InputOptions(),
        position: int | None = None,
    ) -> str:
        metas = []
        for torrent_file_path in meta_file_pathes:
            with open(torrent_file_path, "r") as tf:
                base64.b64encode(tf.read())
        return self._download(
            "aria2.addMetalink",
            id=id,
            params=metas,
            options=options,
            position=position,
        )["result"]

    def remove(
        self,
        gid: str,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.remove",
            id=id,
            params=[gid],
        )["result"]

    def force_remove(
        self,
        gid: str,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.forceRemove",
            id=id,
            params=[gid],
        )["result"]

    def pause(
        self,
        gid: str,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.pause",
            id=id,
            params=[gid],
        )["result"]

    def pause_all(
        self,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.pauseAll",
            id=id,
        )["result"]

    def force_pause(
        self,
        gid: str,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.forcePause",
            params=[gid],
            id=id,
        )["result"]

    def force_pause_all(
        self,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.forcePauseAll",
            id=id,
        )["result"]

    def unpause(
        self,
        gid: str,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.unpause",
            params=[gid],
            id=id,
        )["result"]

    def unpause_all(
        self,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.unpauseAll",
            id=id,
        )["result"]

    def tel_status(
        self,
        gid: str,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.tellStatus",
            params=[gid],
            id=id,
        )["result"]

    def get_uris(
        self,
        gid: str,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.getUris",
            params=[gid],
            id=id,
        )["result"]

    def get_files(
        self,
        gid: str,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.getFiles",
            params=[gid],
            id=id,
        )["result"]

    def get_peers(
        self,
        gid: str,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.getPeers",
            params=[gid],
            id=id,
        )["result"]

    def get_servers(
        self,
        gid: str,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.getServers",
            params=[gid],
            id=id,
        )["result"]

    def tel_active(
        self,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.tellActive",
            id=id,
        )["result"]

    def tel_waiting(
        self,
        offset: int = 0,
        num: int = 100,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.tellWaiting",
            params=[offset, num],
            id=id,
        )["result"]

    def tel_stopped(
        self,
        offset: int = 0,
        num: int = 100,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.tellStopped",
            params=[offset, num],
            id=id,
        )["result"]

    def change_position(
        self,
        gid: str,
        pos: int,
        how: HowPosition,
    ) -> str:
        return self._get(
            "aria2.getServers",
            params=[gid, pos, how.value],
            id=id,
        )["result"]

    def change_uri(
        self,
        gid: str,
        file_index: int,
        del_uris: list[str],
        add_uris: list[str],
    ) -> str:
        return self._get(
            "aria2.getServers",
            params=[gid, file_index, del_uris, add_uris],
            id=id,
        )["result"]

    def get_option(
        self,
        gid: str,
    ) -> dict:
        return self._get(
            "aria2.getOption",
            params=[gid],
            id=id,
        )["result"]

    def change_option(
        self,
        gid: str,
        options: Aria2CommonOptions,
    ) -> str:
        new_options = self.get_option(gid)
        new_options.update(options.to_dict())
        return self._get(
            "aria2.changeOption",
            params=[gid],
            id=id,
        )["result"]

    def get_global_option(
        self,
        id: int | None = None,
    ) -> dict:
        return self._get(
            "aria2.getGlobalOption",
            id=id,
        )["result"]

    def change_global_option(
        self,
        new_global_options: Aria2GlobalOptions,
        id: int | None = None,
    ) -> str:
        global_options = self.get_global_option()
        global_options.update(new_global_options.to_dict())
        return self._get(
            "aria2.changeGlobalOption",
            id=id,
            params=[global_options],
        )["result"]

    def get_global_stat(
        self,
        id: int | None = None,
    ) -> dict:
        return self._get(
            "aria2.getGlobalStat",
            id=id,
        )["result"]

    def purge_download_result(
        self,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.purgeDownloadResult",
            id=id,
        )["result"]

    def get_version(
        self,
        id: int | None = None,
    ) -> str:
        return self._get(
            "system.getVersion",
            id=id,
        )["result"]

    def get_session_info(
        self,
        id: int | None = None,
    ) -> str:
        return self._get(
            "system.getSessionInfo",
            id=id,
        )["result"]

    def shutdown(
        self,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.shutdown",
            id=id,
        )["result"]

    def force_shutdown(
        self,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.forceShutdown",
            id=id,
        )["result"]

    def save_session(
        self,
        id: int | None = None,
    ) -> str:
        return self._get(
            "aria2.saveSession",
            id=id,
        )["result"]

    def list_methods(
        self,
        id: int | None = None,
    ) -> str:
        return self._get(
            "system.listMethods",
            id=id,
        )["result"]

    def list_notifications(
        self,
        id: int | None = None,
    ) -> str:
        return self._get(
            "system.listNotifications",
            id=id,
        )["result"]


if __name__ == "__main__":
    my_aria = Aria2JRPC(
        "http://127.0.0.1:6800",
        secret="shitj",
        global_options=Aria2GlobalOptions(
            max_concurrent_downloads=1,
            continue_=1,
            max_connection_per_server=16,
            split=16,
            dir="d:/",
        ),
    )

    print(my_aria.list_notifications())



from dataclasses import asdict, dataclass
from enum import Enum


class HowPosition(str, Enum):
    POST_SET = "POST_SET"
    POS_CUR = "POS_CUR"
    POS_END = "POS_END"


@dataclass
class Aria2CommonOptions:
    all_proxy: str | None = None
    all_proxy_passwd: str | None = None
    all_proxy_user: str | None = None
    allow_overwrite: str | None = None
    allow_piece_length_change: str | None = None
    always_resume: str | None = None
    async_dns: str | None = None
    auto_file_renaming: str | None = None
    bt_enable_hook_after_hash_check: str | None = None
    bt_enable_lpd: str | None = None
    bt_exclude_tracker: str | None = None
    bt_external_ip: str | None = None
    bt_force_encryption: str | None = None
    bt_hash_check_seed: str | None = None
    bt_load_saved_metadata: str | None = None
    bt_max_peers: str | None = None
    bt_metadata_only: str | None = None
    bt_min_crypto_level: str | None = None
    bt_prioritize_piece: str | None = None
    bt_remove_unselected_file: str | None = None
    bt_request_peer_speed_limit: str | None = None
    bt_require_crypto: str | None = None
    bt_save_metadata: str | None = None
    bt_seed_unverified: str | None = None
    bt_stop_timeout: str | None = None
    bt_tracker: str | None = None
    bt_tracker_connect_timeout: str | None = None
    bt_tracker_interval: str | None = None
    bt_tracker_timeout: str | None = None
    check_integrity: str | None = None
    conditional_get: str | None = None
    connect_timeout: str | None = None
    content_disposition_default_utf8: str | None = None
    continue_: str | None = None
    dir: str | None = None
    enable_http_keep_alive: str | None = None
    enable_http_pipelining: str | None = None
    enable_mmap: str | None = None
    enable_peer_exchange: str | None = None
    file_allocation: str | None = None
    follow_metalink: str | None = None
    follow_torrent: str | None = None
    force_save: str | None = None
    ftp_passwd: str | None = None
    ftp_pasv: str | None = None
    ftp_proxy: str | None = None
    ftp_proxy_passwd: str | None = None
    ftp_proxy_user: str | None = None
    ftp_reuse_connection: str | None = None
    ftp_type: str | None = None
    ftp_user: str | None = None
    gid: str | None = None
    hash_check_only: str | None = None
    header: str | None = None
    http_accept_gzip: str | None = None
    http_auth_challenge: str | None = None
    http_no_cache: str | None = None
    http_passwd: str | None = None
    http_proxy: str | None = None
    http_proxy_passwd: str | None = None
    http_proxy_user: str | None = None
    http_user: str | None = None
    https_proxy: str | None = None
    https_proxy_passwd: str | None = None
    https_proxy_user: str | None = None
    lowest_speed_limit: str | None = None
    max_connection_per_server: str | None = None
    max_download_limit: str | None = None
    max_file_not_found: str | None = None
    max_mmap_limit: str | None = None
    max_resume_failure_tries: str | None = None
    max_tries: str | None = None
    max_upload_limit: str | None = None
    metalink_enable_unique_protocol: str | None = None
    metalink_language: str | None = None
    metalink_location: str | None = None
    metalink_os: str | None = None
    metalink_preferred_protocol: str | None = None
    metalink_version: str | None = None
    min_split_size: str | None = None
    no_file_allocation_limit: str | None = None
    no_netrc: str | None = None
    no_proxy: str | None = None
    pause_metadata: str | None = None
    proxy_method: str | None = None
    realtime_chunk_checksum: str | None = None
    referer: str | None = None
    remote_time: str | None = None
    remove_control_file: str | None = None
    retry_wait: str | None = None
    reuse_uri: str | None = None
    seed_ratio: str | None = None
    seed_time: str | None = None
    split: str | None = None
    ssh_host_key_md: str | None = None
    stream_piece_selector: str | None = None
    timeout: str | None = None
    uri_selector: str | None = None
    use_head: str | None = None
    user_agent: str | None = None

    def to_dict(self) -> dict:
        data = asdict(self)
        data["continue"] = data.pop("continue_")
        data = {
            key.replace("_", "-"): str(value)
            for key, value in asdict(self).items()
            if value is not None
        }
        return data


@dataclass
class Aria2BaseOptions(Aria2CommonOptions):
    all_proxy: str | None = None
    all_proxy_passwd: str | None = None
    all_proxy_user: str | None = None
    allow_overwrite: str | None = None
    allow_piece_length_change: str | None = None
    always_resume: str | None = None
    async_dns: str | None = None
    auto_file_renaming: str | None = None
    bt_enable_hook_after_hash_check: str | None = None
    bt_enable_lpd: str | None = None
    bt_exclude_tracker: str | None = None
    bt_external_ip: str | None = None
    bt_force_encryption: str | None = None
    bt_hash_check_seed: str | None = None
    bt_load_saved_metadata: str | None = None
    bt_max_peers: str | None = None
    bt_metadata_only: str | None = None
    bt_min_crypto_level: str | None = None
    bt_prioritize_piece: str | None = None
    bt_remove_unselected_file: str | None = None
    bt_request_peer_speed_limit: str | None = None
    bt_require_crypto: str | None = None
    bt_save_metadata: str | None = None
    bt_seed_unverified: str | None = None
    bt_stop_timeout: str | None = None
    bt_tracker: str | None = None
    bt_tracker_connect_timeout: str | None = None
    bt_tracker_interval: str | None = None
    bt_tracker_timeout: str | None = None
    check_integrity: str | None = None
    conditional_get: str | None = None
    connect_timeout: str | None = None
    content_disposition_default_utf8: str | None = None
    continue_: str | None = None
    dir: str | None = None
    dry_run: str | None = None
    enable_http_keep_alive: str | None = None
    enable_http_pipelining: str | None = None
    enable_mmap: str | None = None
    enable_peer_exchange: str | None = None
    file_allocation: str | None = None
    follow_metalink: str | None = None
    follow_torrent: str | None = None
    force_save: str | None = None
    ftp_passwd: str | None = None
    ftp_pasv: str | None = None
    ftp_proxy: str | None = None
    ftp_proxy_passwd: str | None = None
    ftp_proxy_user: str | None = None
    ftp_reuse_connection: str | None = None
    ftp_type: str | None = None
    ftp_user: str | None = None
    gid: str | None = None
    hash_check_only: str | None = None
    header: str | None = None
    http_accept_gzip: str | None = None
    http_auth_challenge: str | None = None
    http_no_cache: str | None = None
    http_passwd: str | None = None
    http_proxy: str | None = None
    http_proxy_passwd: str | None = None
    http_proxy_user: str | None = None
    http_user: str | None = None
    https_proxy: str | None = None
    https_proxy_passwd: str | None = None
    https_proxy_user: str | None = None
    lowest_speed_limit: str | None = None
    max_connection_per_server: str | None = None
    max_download_limit: str | None = None
    max_file_not_found: str | None = None
    max_mmap_limit: str | None = None
    max_resume_failure_tries: str | None = None
    max_tries: str | None = None
    max_upload_limit: str | None = None
    metalink_base_uri: str | None = None
    metalink_enable_unique_protocol: str | None = None
    metalink_language: str | None = None
    metalink_location: str | None = None
    metalink_os: str | None = None
    metalink_preferred_protocol: str | None = None
    metalink_version: str | None = None
    min_split_size: str | None = None
    no_file_allocation_limit: str | None = None
    no_netrc: str | None = None
    no_proxy: str | None = None
    parameterized_uri: str | None = None
    pause_metadata: str | None = None
    piece_length: str | None = None
    proxy_method: str | None = None
    realtime_chunk_checksum: str | None = None
    referer: str | None = None
    remote_time: str | None = None
    remove_control_file: str | None = None
    retry_wait: str | None = None
    reuse_uri: str | None = None
    rpc_save_upload_metadata: str | None = None
    seed_ratio: str | None = None
    seed_time: str | None = None
    split: str | None = None
    ssh_host_key_md: str | None = None
    stream_piece_selector: str | None = None
    timeout: str | None = None
    uri_selector: str | None = None
    use_head: str | None = None
    user_agent: str | None = None


@dataclass
class Aria2InputOptions(Aria2BaseOptions):
    checksum: str | None = None
    index_out: str | None = None
    out: str | None = None
    pause: str | None = None
    select_file: str | None = None


@dataclass
class Aria2GlobalOptions(Aria2BaseOptions):
    bt_max_open_files: str | None = None
    download_result: str | None = None
    keep_unfinished_download_result: str | None = None
    log: str | None = None
    log_level: str | None = None
    max_concurrent_downloads: str | None = None
    max_download_result: str | None = None
    max_overall_download_limit: str | None = None
    max_overall_upload_limit: str | None = None
    optimize_concurrent_downloads: str | None = None
    save_cookies: str | None = None
    save_session: str | None = None
    server_stat_of: str | None = None
